use enum_display::EnumDisplay;
use serde::{Deserialize, Serialize};

#[derive(Deserialize, Serialize, Debug, Clone, Copy, EnumDisplay)]
pub enum Record {
    TXT,
    A,
    AAAA,
    MX,
    ANY,
    CAA,
    CNAME,
    DNSKEY,
    DS,
    NS,
    PTR,
    SOA,
    SRV,
    TLSA,
    TSIG,
}

#[derive(Deserialize, Serialize, Debug, Clone)]
pub struct RRSet {
    #[serde(rename = "type")]
    pub record: Record,
    pub domain: String,
    #[serde(deserialize_with = "empty_string_as_none")]
    pub subname: Option<String>,
    pub name: String,
    pub ttl: i32,
    pub records: Vec<String>,
    pub created: String,
    pub touched: String,
}

use serde::de::IntoDeserializer;

fn empty_string_as_none<'de, D, T>(de: D) -> Result<Option<T>, D::Error>
where
    D: serde::Deserializer<'de>,
    T: serde::Deserialize<'de>,
{
    let opt = Option::<String>::deserialize(de)?;
    let opt = opt.as_ref().map(String::as_str);
    match opt {
        None | Some("") => Ok(None),
        Some(s) => T::deserialize(s.into_deserializer()).map(Some),
    }
}

#[derive(Deserialize, Serialize, Debug, Clone, Default)]
pub struct RRSetPatch {
    #[serde(alias = "type")]
    #[serde(skip_serializing_if = "Option::is_none")]
    pub record: Option<Record>,
    #[serde(skip_serializing_if = "Option::is_none")]
    pub domain: Option<String>,
    #[serde(skip_serializing_if = "Option::is_none")]
    pub subname: Option<String>,
    #[serde(skip_serializing_if = "Option::is_none")]
    pub name: Option<String>,
    #[serde(skip_serializing_if = "Option::is_none")]
    pub ttl: Option<i32>,
    #[serde(skip_serializing_if = "Option::is_none")]
    pub records: Option<Vec<String>>,
    #[serde(skip_serializing_if = "Option::is_none")]
    pub created: Option<String>,
    #[serde(skip_serializing_if = "Option::is_none")]
    pub touched: Option<String>,
}
