use std::vec;

use async_trait::async_trait;
use quick_xml::{
    events::{BytesEnd, BytesStart, Event},
    Writer,
};
use tokio::io::AsyncWrite;

use crate::{error::StreamError, tag};

pub struct Feature {
    name: &'static str,
    required: bool,
    namespace: Option<&'static str>,
}

#[async_trait]
impl tag::Tag for Feature {
    async fn write_tag<W>(&self, writer: W) -> Result<(), StreamError>
    where
        W: AsyncWrite + Unpin + Send,
    {
        let mut writer = Writer::new(writer);
        writer
            .write_event_async(Event::Start(BytesStart::new(self.name).with_attributes(
                if let Some(namespace) = self.namespace {
                    vec![("xmlns", namespace)]
                } else {
                    vec![]
                },
            )))
            .await?;
        if self.required {
            writer
                .write_event_async(Event::Empty(BytesStart::new("required")))
                .await?;
        }
        writer
            .write_event_async(Event::End(BytesEnd::new(self.name)))
            .await?;

        Ok(())
    }
}

impl Feature {
    pub const fn start_tls(required: bool) -> Feature {
        Feature {
            required,
            name: "starttls",
            namespace: Some(tag::TLS_NAMESPACE),
        }
    }
}
