use async_trait::async_trait;
use quick_xml::{
    events::{BytesEnd, BytesStart, Event},
    Writer,
};
use tokio::io::AsyncWrite;

use crate::error::StreamError;

pub const STREAM_ELEMENT_NAME: &[u8] = b"stream:stream";
pub const ERROR_ELEMENT: &str = "stream:error";
pub const TLS_NAMESPACE: &str = "urn:ietf:params:xml:ns:xmpp-tls";
pub const STARTTLS: &[u8] = b"starttls";
pub const FEATURE: &str = "stream:features";

pub struct HollowTag<'a> {
    name: &'a str,
    namespace: &'static str,
}

impl<'a> From<(&'a str, &'static str)> for HollowTag<'a> {
    fn from((name, namespace): (&'a str, &'static str)) -> Self {
        Self { name, namespace }
    }
}

#[async_trait]
pub trait Tag {
    async fn write_tag<W>(&self, writer: W) -> Result<(), StreamError>
    where
        W: AsyncWrite + Unpin + Send;
}

#[async_trait]
impl<'a> Tag for HollowTag<'a> {
    async fn write_tag<W>(&self, writer: W) -> Result<(), StreamError>
    where
        W: AsyncWrite + Unpin + Send,
    {
        let mut writer = Writer::new(writer);
        writer
            .write_event_async(Event::Start(
                BytesStart::new(self.name).with_attributes(vec![("xmlns", self.namespace)]),
            ))
            .await?;
        writer
            .write_event_async(Event::End(BytesEnd::new(self.name)))
            .await?;

        Ok(())
    }
}
